VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "cQueue"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Enumeration for possible queue types
Public Enum eQueueType
    qtFIFO = 0
    qtLIFO = 1
    qtPriority = 2
End Enum

Private Local_oCollection As Collection ' Local Collection object
Private Local_nQueueType As eQueueType  ' Local Copy of queue type

' Property Access to queue type
Public Property Let QueueType(nType As eQueueType)
    Local_nQueueType = nType
End Property
Public Property Get QueueType() As eQueueType
    QueueType = Local_nQueueType
End Property

' InQueue property: Check if item is in queue
Public Function InQueue(nIdentity As Long) As Boolean
    On Error GoTo E_InQueue
    InQueue = True
    If Local_oCollection.Item("I" + Format(nIdentity, "000000")) Is Nothing Then
        InQueue = False
    End If
    Exit Function
E_InQueue:
    InQueue = False
End Function
' SetPriority method: Sets a new priority for an element already in list
Public Sub SetPriority(nIdentity As Long, nPriority As Long)
    
    On Error GoTo E_SetPriority
    If Not Local_oCollection.Item("I" + Format(nIdentity, "000000")) Is Nothing Then
        
        Dim oPQMember As cPQEntry   ' Variable to hold priority queue member
        Dim nRun As Integer         ' Variable to run through queue entries
        
        ' Store away Priority queue member with new priority, remove member from queue
        Set oPQMember = Local_oCollection.Item("I" + Format(nIdentity, "000000"))
        oPQMember.Priority = nPriority
        Local_oCollection.Remove "I" + Format(nIdentity, "000000")
        
        ' Look for appropriate place
        For nRun = 1 To Local_oCollection.Count
            If Local_oCollection.Item(nRun).Priority < nPriority Then Exit For
        Next
        
        ' Insert member
        If Local_oCollection.Count < nRun Then
            Local_oCollection.Add oPQMember, "I" + Format(nIdentity, "000000")
        Else
            Local_oCollection.Add oPQMember, "I" + Format(nIdentity, "000000"), , nRun
        End If

    
    End If
    
    Exit Sub
    
E_SetPriority:
   
End Sub
' Push method: Push an element on the queue
Public Sub Push(oNew As Object, Optional nIdentity As Long, Optional ByVal nPriority As Long)
    
    ' For priority queues, add element at appropriate place in queue
    If Local_nQueueType = qtPriority Then
        
        Dim oPQMember As cPQEntry   ' Variable to hold new priority queue member
        Dim nRun As Integer         ' Variable to run through queue entries
        
        ' Create Priority queue member
        Set oPQMember = New cPQEntry
        Set oPQMember.Member = oNew
        oPQMember.Priority = nPriority
        
        ' Look for appropriate place
        For nRun = 1 To Local_oCollection.Count
            If Local_oCollection.Item(nRun).Priority < nPriority Then Exit For
        Next
        
        ' Insert member
        If Local_oCollection.Count < nRun Then
            Local_oCollection.Add oPQMember, "I" + Format(nIdentity, "000000")
        Else
            Local_oCollection.Add oPQMember, "I" + Format(nIdentity, "000000"), , nRun
        End If
        
    Else
    
    ' For LIFO or FIFO queues, add element at end of list
        If IsMissing(nIdentity) Or nIdentity = 0 Then
            Local_oCollection.Add oNew
        Else
            Local_oCollection.Add oNew, "I" + Format(nIdentity, "000000")
        End If
    
    End If
    Debug.Print Local_oCollection.Count
End Sub

' Pop method: Pop an element from the queue
Public Function Pop() As Object

    ' Return object according to queue type settings, return nothing if queue is empty
    If Local_oCollection.Count > 0 Then
        
        Select Case Local_nQueueType
            
            ' For FIFO, the element to pop is the first one, for it has been added the longest time ago
            Case qtFIFO
                Set Pop = Local_oCollection.Item(1)
                Local_oCollection.Remove 1
            
            ' For LIFO, the element to pop is the last one, for it has been added most recently
            Case qtLIFO
                Set Pop = Local_oCollection.Item(Local_oCollection.Count)
                Local_oCollection.Remove Local_oCollection.Count
            
            ' For Priority, the element to pop is the last one, for it has the highest index (priority)
            Case qtPriority
                Set Pop = Local_oCollection.Item(Local_oCollection.Count).Member
                Local_oCollection.Remove Local_oCollection.Count
                
        End Select
        
    Else
        Set Pop = Nothing
    End If
    
End Function

' Remove method: Remove an element from the queue
Public Sub Remove(nIdentity As Long)
    On Error GoTo E_Remove
    Local_oCollection.Remove "I" + Format(nIdentity, "000000")
E_Remove:
End Sub

' Return empty status
Public Function IsEmpty() As Boolean
    IsEmpty = (Local_oCollection.Count = 0)
End Function

' Create queue on initialisation
Private Sub Class_Initialize()

    Set Local_oCollection = New Collection
    
End Sub

' Drop queue on termination
Private Sub Class_Terminate()

    Set Local_oCollection = Nothing
    
End Sub

