VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "cSquare"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
Option Explicit

' Enumeration for state in path search
Public Enum ePathState
    psNone = 0
    psVisited = 1
    psPathed = 2
End Enum

' Enumeration for square type
Public Enum eSquareType
    stDefault = 0
    stStart = 1
    stDestination = 2
End Enum

' Enumeration for mode of neighbour matching
Public Enum eMatchMode
    mmRaise = 0
    mmLower = 1
End Enum

' API drawing Functions
Private Declare Function Rectangle Lib "gdi32" (ByVal hdc As Long, ByVal X1 As Long, ByVal Y1 As Long, ByVal X2 As Long, ByVal Y2 As Long) As Long
Private Declare Function SetPixelV Lib "gdi32" (ByVal hdc As Long, ByVal X As Long, ByVal Y As Long, ByVal crColor As Long) As Long

' Standard variables every map should include
Private Local_nX As Long                    ' Horizontal position of square
Private Local_nY As Long                    ' Vertical position of square
Private Local_nAltitude As Long             ' Movement costs on this square
Private Local_oParent As cSquares           ' Local copy of parent collection
Private Local_nSquareType As eSquareType    ' Local copy of square type

' Variables included for pathfinding        '
Private Local_nPathState As ePathState      ' Local copy of pathstate
Private Local_nCost As Long                 ' Local accumulated cost for use with djikstra and astar
Private Local_nCostEstimate As Long         ' Heuristic cost estimation for use with astar
Private Local_oParentSquare As cSquare      ' Local copy of parent square
Private Local_sKey As String                ' Local copy of key of square


' Property handling for parent object
Public Property Get Parent() As cSquares
    Set Parent = Local_oParent
End Property
Public Property Set Parent(oParent As cSquares)
    Set Local_oParent = oParent
End Property

' Property handling for parent square object
Public Property Get ParentSquare() As cSquare
    Set ParentSquare = Local_oParentSquare
End Property
Public Property Set ParentSquare(oParent As cSquare)
    Set Local_oParentSquare = oParent
End Property

' Property handling for key
Public Property Let Key(sKey As String)
    Local_sKey = sKey
End Property
Public Property Get Key() As String
    Key = Local_sKey
End Property

' Property handling for distance estimate
Public Property Let CostEstimate(nCostEstimate As Long)
    Local_nCostEstimate = nCostEstimate
End Property
Public Property Get CostEstimate() As Long
    CostEstimate = Local_nCostEstimate
End Property


' Property handling for accumulated cost value
Public Property Let Cost(nCost As Long)
    Local_nCost = nCost
End Property
Public Property Get Cost() As Long
    Cost = Local_nCost
End Property

' Property handling for square type
Public Property Let SquareType(nSquareType As eSquareType)
    Local_nSquareType = nSquareType
    If nSquareType = stStart Then Set Local_oParent.ItemStart = Me
    If nSquareType = stDestination Then Set Local_oParent.ItemDestination = Me
End Property
Public Property Get SquareType() As eSquareType
    SquareType = Local_nSquareType
End Property

' Property handling for identity property
Public Function Identity() As String
    Identity = "I" + Format(Local_nX + Local_nY * 40, "00000")
End Function

Public Property Let X(nX As Long)
    Local_nX = nX
End Property
Public Property Get X() As Long
    X = Local_nX
End Property
Public Property Let Y(nY As Long)
    Local_nY = nY
End Property
Public Property Get Y() As Long
    Y = Local_nY
End Property
Public Property Let Altitude(nAltitude As Long)
    Local_nAltitude = nAltitude
End Property
Public Property Get Altitude() As Long
    Altitude = Local_nAltitude
End Property
Public Property Let Pathstate(nPathState As ePathState)
    Local_nPathState = nPathState
End Property
Public Property Get Pathstate() As ePathState
    Pathstate = Local_nPathState
End Property

' Method for retrieving neighbour squares
Public Function Neighbour(ByVal nDirection As Byte) As cSquare
        
    Select Case nDirection
        Case 0
            If Local_nY > 0 Then
                Set Neighbour = Local_oParent.Item(Local_nX + (Local_nY - 1) * 40)
            Else
                Set Neighbour = Nothing
            End If
        Case 1
            If Local_nY > 0 And Local_nX < 39 Then
                Set Neighbour = Local_oParent.Item(Local_nX + 1 + (Local_nY - 1) * 40)
            Else
                Set Neighbour = Nothing
            End If
        Case 2
            If Local_nX < 39 Then
                Set Neighbour = Local_oParent.Item(Local_nX + 1 + Local_nY * 40)
            Else
                Set Neighbour = Nothing
            End If
        Case 3
            If Local_nY < 39 And Local_nX < 39 Then
                Set Neighbour = Local_oParent.Item(Local_nX + 1 + (Local_nY + 1) * 40)
            Else
                Set Neighbour = Nothing
            End If
        Case 4
            If Local_nY < 39 Then
                Set Neighbour = Local_oParent.Item(Local_nX + (Local_nY + 1) * 40)
            Else
                Set Neighbour = Nothing
            End If
        Case 5
            If Local_nY < 39 And Local_nX > 0 Then
                Set Neighbour = Local_oParent.Item(Local_nX - 1 + (Local_nY + 1) * 40)
            Else
                Set Neighbour = Nothing
            End If
        Case 6
            If Local_nX > 0 Then
                Set Neighbour = Local_oParent.Item(Local_nX - 1 + (Local_nY) * 40)
            Else
                Set Neighbour = Nothing
            End If
        Case 7
            If Local_nY > 0 And Local_nX > 0 Then
                Set Neighbour = Local_oParent.Item(Local_nX - 1 + (Local_nY - 1) * 40)
            Else
                Set Neighbour = Nothing
            End If
    End Select
        
End Function

' Redraw method: Draw square to screen
Public Sub Redraw()

    ' Create color for drawing
    fPaths.picDisplay.FillColor = Local_nAltitude * 1310720 + 3285024
    
    ' Draw square
    Rectangle fPaths.picDisplay.hdc, Local_nX * 10, Local_nY * 10, Local_nX * 10 + 11, Local_nY * 10 + 11
    
    ' Draw path state
    Select Case Local_nPathState
        Case psVisited
            SetPixelV fPaths.picDisplay.hdc, Local_nX * 10 + 5, Local_nY * 10 + 5, RGB(96, 96, 96)
        Case psPathed
            SetPixelV fPaths.picDisplay.hdc, Local_nX * 10 + 5, Local_nY * 10 + 5, RGB(255, 255, 255)
    End Select
    
    ' Draw special (Start, Destination)
    If Not Local_nSquareType = stDefault Then
        fPaths.picDisplay.FillColor = IIf(Local_nSquareType = stStart, RGB(32, 255, 32), RGB(255, 32, 32))
        Rectangle fPaths.picDisplay.hdc, Local_nX * 10 + 2, Local_nY * 10 + 2, Local_nX * 10 + 9, Local_nY * 10 + 9
    End If
    
End Sub

' Execute recursive matching
    
    ' This could also have been done using a queued algorithm.
    ' But because most of the pathfinding algorithms are using
    ' queues, recursion is used here to demonstrate as many
    ' different techniques as possible.
    
Public Sub MatchNeighbours(ByVal nMode As eMatchMode)
    
    Dim nRun As Byte
    
    ' Step through all neigbour squares
    For nRun = 0 To 7
        
        ' Check for neighbour existance
        If Not Neighbour(nRun) Is Nothing Then
        
            With Neighbour(nRun)
                
                ' Mode is lower: Lower square, recurse
                If nMode = mmLower Then
                    If .Altitude > Local_nAltitude + 1 Then
                        .Altitude = Local_nAltitude + 1
                        .MatchNeighbours nMode
                    End If
                End If
            
                ' Mode is raise: Raise square, recurse
                If nMode = mmRaise Then
                    If .Altitude < Local_nAltitude - 1 Then
                        .Altitude = Local_nAltitude - 1
                        .MatchNeighbours nMode
                    End If
                End If
                
                ' Redraw square
                .Redraw
                
            End With
        End If
    Next
    
End Sub

Public Function DestinationDistanceEstimate() As Long
    Dim nDX As Long, nDY As Long
    nDX = Local_nX - Local_oParent.ItemDestination.X
    nDY = Local_nY - Local_oParent.ItemDestination.Y
    DestinationDistanceEstimate = Int(Sqr(nDX ^ 2 + nDY ^ 2))
End Function
