Attribute VB_Name = "mAlgorithms"
Option Explicit

           
' Pathfinding with breath search
Sub A_Breath()

    Dim oCurrent As cSquare     ' Master square for processing
    Dim oSibling As cSquare     ' Subordinate square for processing
    Dim oStart As cSquare       ' Square to start with
    Dim oQueue As LinearQueue   ' Queue for squares to store in
    Dim nRun As Integer         ' Variable to run through squares
    
    ' Create a new FIFO queue
    Set oQueue = New LinearQueue
    oQueue.QueueType = qtFIFO
    
    ' Initializie start square
    Set oStart = oSquares.ItemStart
    Set oStart.ParentSquare = Nothing
    oStart.Pathstate = psVisited
    
    ' Push start square on open list
    oQueue.Push oStart
    
    ' Repeat until queue is empty
    Do While Not oQueue.IsEmpty
    
        ' Pop square to be processed from the queue
        Set oCurrent = oQueue.Pop
        
        ' Check if popped square is destination square
        If oCurrent.Identity = oSquares.ItemDestination.Identity Then
            ' Return path if destination reached
            Do While Not oCurrent.ParentSquare Is Nothing
                oCurrent.Pathstate = psPathed
                oCurrent.Redraw
                Set oCurrent = oCurrent.ParentSquare
            Loop
            Set oQueue = Nothing
            Exit Sub
            
        End If
        
        ' Run throuhg all neighbours of current square
        For nRun = 0 To 7
            Set oSibling = oCurrent.Neighbour(nRun)
            
            ' If neighbour exists...
            If Not oSibling Is Nothing Then
            
                ' if neighbour is valid path
                If Not (Abs(oCurrent.Altitude - oSibling.Altitude) > nHeightDifference) Then
    
                    ' if square has not yet been visited, push it on the queue
                    If Not oSibling.Pathstate = psVisited Then
                        Set oSibling.ParentSquare = oCurrent
                        oSibling.Pathstate = psVisited
                        oQueue.Push oSibling
                    End If
                    
                End If
            End If
        Next
        
        ' Redraw square on map
        oCurrent.Redraw

    Loop
    
    Set oQueue = Nothing
    
End Sub
            
' Pathfinding with Dijkstra's algorithm
Sub A_Dijkstra()

    Dim oCurrent As cSquare     ' Master square for processing
    Dim oSibling As cSquare     ' Subordinate square for processing
    Dim oStart As cSquare       ' Square to start with
    Dim oQueue As PriorityQueue ' Queue for squares to store in
    Dim nRun As Integer         ' Variable to run through squares
    Dim nCost As Long           ' Cost to new square
    
    ' Create a new Priority queue
    Set oQueue = New PriorityQueue
    
    ' Initializie start square
    Set oStart = oSquares.ItemStart
    Set oStart.ParentSquare = Nothing
    oStart.Cost = 1
    oStart.Pathstate = psVisited
    
    ' Push start square on open list
    oQueue.Push oStart, oStart.Cost, oStart.Identity

    ' Repeat until queue is empty
    Do While Not oQueue.IsEmpty

        ' Pop square to be processed from the queue
        Set oCurrent = oQueue.Pop(pmLeast)

        ' Check if popped square is destination square
        If oCurrent.Identity = oSquares.ItemDestination.Identity Then
            ' Return path if destination reached
            Do While Not oCurrent.ParentSquare Is Nothing
                oCurrent.Pathstate = psPathed
                oCurrent.Redraw
                Set oCurrent = oCurrent.ParentSquare
            Loop
            Set oQueue = Nothing
            Exit Sub
        End If

        ' Run throuhg all neighbours of current square
        For nRun = 0 To 7
            Set oSibling = oCurrent.Neighbour(nRun)

            ' If neighbour exists...
            If Not oSibling Is Nothing Then
            
                ' If neighbour not yet visited ...
                If oSibling.Pathstate = psNone Then
                
                    ' Only allow movement if not to steep ...
                    If Not (Abs(oCurrent.Altitude - oSibling.Altitude) > nHeightDifference) Then
                        ' Calculate cost for movement, diagonal movement counts more by a factor
                        nCost = Int(oCurrent.Cost + (1 + Abs(oCurrent.Altitude - oSibling.Altitude)) * (IIf(nRun Mod 2 = 0, 1, nDiagonalMultiplyer)) * 10)
                        
                        ' if square has not yet been queued ...
                        If oQueue.Item(oSibling.Identity) Is Nothing Then
                            ' ... add square to queue
                            Set oSibling.ParentSquare = oCurrent
                            oSibling.Cost = nCost
                            oQueue.Push oSibling, nCost, oSibling.Identity
                        Else
                        ' Square is already in queue ...
                            If oSibling.Cost > nCost Then
                                ' Update square, if better cost from current parent square ...
                                Set oSibling.ParentSquare = oCurrent
                                oSibling.Cost = nCost
                                oQueue.Update oSibling.Identity, nCost
                            End If
                        End If
                    End If
                End If
                
            End If
        Next
        
        ' Redraw square on map
        oCurrent.Pathstate = psVisited
        oCurrent.Redraw

    Loop
    
    Set oQueue = Nothing

End Sub
            
' Pathfinding with the AStart algorithm
Sub A_Astar()

    Dim oCurrent As cSquare         ' Master square for processing
    Dim oSibling As cSquare         ' Subordinate square for processing
    Dim oStart As cSquare           ' Square to start with
    Dim oQueue As PriorityQueue     ' Queue for open squares to store in
    Dim nRun As Integer             ' Variable to run through squares
    Dim nCost As Long               ' Cost to new square
    
    ' Create new Priority queues for the open and closed list
    Set oQueue = New PriorityQueue
    
    ' Initializie start square
    Set oStart = oSquares.ItemStart
    Set oStart.ParentSquare = Nothing
    oStart.Cost = 1
    oStart.CostEstimate = Int(oStart.DestinationDistanceEstimate * 10 * nDiagonalMultiplyer * 10)
    oStart.Pathstate = psVisited
    
    ' Push start square on open list
    oQueue.Push oStart, oStart.Cost + oStart.CostEstimate, oStart.Identity

    ' Repeat until queue is empty
    Do While Not oQueue.IsEmpty

        ' Pop square to be processed from the queue
        Set oCurrent = oQueue.Pop(pmLeast)

        ' Check if popped square is destination square
        If oCurrent.Identity = oSquares.ItemDestination.Identity Then
            ' Return path if destination reached
            Do While Not oCurrent.ParentSquare Is Nothing
                oCurrent.Pathstate = psPathed
                oCurrent.Redraw
                Set oCurrent = oCurrent.ParentSquare
            Loop
            Set oQueue = Nothing
            Exit Sub
        End If

        ' Run throuhg all neighbours of current square
        For nRun = 0 To 7
            Set oSibling = oCurrent.Neighbour(nRun)

            ' If neighbour exists...
            If Not oSibling Is Nothing Then
            
                ' Only allow movement if not to steep ...
                If Not (Abs(oCurrent.Altitude - oSibling.Altitude) > nHeightDifference) Then
                
                    ' Calculate cost for movement, diagonal movement counts more by a factor
                    nCost = Int(oCurrent.Cost + (1 + Abs(oCurrent.Altitude - oSibling.Altitude)) * (IIf(nRun Mod 2 = 0, 1, nDiagonalMultiplyer)) * 10)
                
                    ' If neighbour not yet visited ...
                    If oSibling.Pathstate = psNone Then
                        ' If square has not yet been queued ...
                        If oQueue.Item(oSibling.Identity) Is Nothing Then
                            ' ... add square to queue
                            Set oSibling.ParentSquare = oCurrent
                            oSibling.Cost = nCost
                            oSibling.CostEstimate = Int(oSibling.DestinationDistanceEstimate * 10 * nDiagonalMultiplyer * 10)
                            oQueue.Push oSibling, nCost + oSibling.CostEstimate, oSibling.Identity
                        Else
                        ' Square is already in queue ...
                            If oSibling.Cost > nCost Then
                                ' Update square, if better cost from current parent square ...
                                Set oSibling.ParentSquare = oCurrent
                                oSibling.Cost = nCost
                                oQueue.Update oSibling.Identity, nCost
                                oSibling.Pathstate = psNone
                            End If
                        End If
                    End If
                End If
            End If
        Next
        
        ' Redraw square on map
        oCurrent.Pathstate = psVisited
        oCurrent.Redraw

    Loop
    
    Set oQueue = Nothing

End Sub
